#!/usr/bin/env python
# -*- coding: utf-8 -*-

# noipy.dnsupdater
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

from __future__ import print_function

import re

import requests

from . import __title__, __version__, __email__

AVAILABLE_PLUGINS = {
    'noip': 'NoipDnsUpdater',
    'dyn': 'DynDnsUpdater',
    'duck': 'DuckDnsUpdater',
    'generic': 'GenericDnsUpdater',
}

DEFAULT_PLUGIN = 'generic'


response_messages = {
    'OK': "SUCCESS: DNS hostname successfully updated.",
    'badauth': "ERROR: Invalid username or password (badauth).",
    '401': "ERROR: Invalid username or password (401).",
    '403': "ERROR: Invalid username or password (403).",
    '!donator': "ERROR: Update request include a feature that is not available to informed user.",
    'notfqdn': "ERROR: The hostname specified is not a fully-qualified domain name "
               "(not in the form hostname.dyndns.org or domain.com).",
    'nohost': "ERROR: Hostname specified does not exist in this user account.",
    'numhost': "ERROR: Too many hosts (more than 20) specified in an update. "
               "Also returned if trying to update a round robin (which is not allowed).",
    'abuse': "ERROR: Username/hostname is blocked due to update abuse.",
    'badagent': "ERROR: User agent not sent or HTTP method not permitted.",
    'dnserr': "ERROR: DNS error encountered.",
    '911': "ERROR: Problem on server side. Retry update in a few minutes.",
    'KO': "ERROR: Hostname and/or token incorrect.",
}


class DnsUpdaterPlugin(object):
    """ Base class for any DDNS updater
    """

    auth_type = ""

    def __init__(self, auth, hostname, options=None):
        """Init plugin with auth information, hostname and IP address.
        """

        self._auth = auth
        self._hostname = hostname
        self._options = {} if options is None else options
        self.last_ddns_response = ""

    @property
    def auth(self):
        return self._auth

    @property
    def hostname(self):
        return self._hostname

    @property
    def _base_url(self):
        """Get the base URL for DDNS Update API. URL must contain 'hostname'
        and 'ip'. If authentication is via token string 'token' argument must
        be provided as well. Example:
          http://{auth_str}@ddnsprovider.com/update?hostname={hostname}&ip={ip}

        This method must be implemented by plugin subclasses
        """
        pass

    def update_dns(self, new_ip):
        """Call No-IP API based on dict login_info and return the status code.
        """

        headers = None
        if self.auth_type == 'T':
            api_call_url = self._base_url.format(hostname=self.hostname,
                                                 token=self.auth.token,
                                                 ip=new_ip)
        else:
            api_call_url = self._base_url.format(hostname=self.hostname,
                                                 ip=new_ip)
            headers = {
                'Authorization': "Basic %s" %
                                 self.auth.base64key.decode('utf-8'),
                'User-Agent': "%s/%s %s" % (__title__, __version__, __email__)
            }

        r = requests.get(api_call_url, headers=headers)
        self.last_ddns_response = str(r.text).strip()

        return r.status_code, r.text

    @property
    def status_message(self):
        """Return friendly response from API based on response code. """

        msg = None
        if self.last_ddns_response in response_messages.keys():
            return response_messages.get(self.last_ddns_response)

        if 'good' in self.last_ddns_response:
            ip = re.search(r'(\d{1,3}\.?){4}', self.last_ddns_response).group()
            msg = "SUCCESS: DNS hostname IP (%s) successfully updated." % ip
        elif 'nochg' in self.last_ddns_response:
            ip = re.search(r'(\d{1,3}\.?){4}', self.last_ddns_response).group()
            msg = "SUCCESS: IP address (%s) is up to date, nothing was changed. " \
                  "Additional 'nochg' updates may be considered abusive." % ip
        else:
            msg = "ERROR: Ooops! Something went wrong !!!"

        return msg

    def __str__(self):
        return '%s(host=%s)' % (type(self).__name__, self.hostname)


class NoipDnsUpdater(DnsUpdaterPlugin):
    """No-IP DDNS provider plugin """

    auth_type = "P"

    @property
    def _base_url(self):
        return "https://dynupdate.no-ip.com/nic/update?hostname={hostname}" \
               "&myip={ip}"


class DynDnsUpdater(DnsUpdaterPlugin):
    """DynDNS DDNS provider plugin """

    auth_type = "P"

    @property
    def _base_url(self):
        return "http://members.dyndns.org/nic/update?hostname={hostname}" \
               "&myip={ip}&wildcard=NOCHG&mx=NOCHG&backmx=NOCHG"


class DuckDnsUpdater(DnsUpdaterPlugin):
    """DuckDNS DDNS provider plugin """

    auth_type = "T"

    @property
    def hostname(self):
        hostname = self._hostname
        found = re.search(r'(.*?)\.duckdns\.org', self._hostname)
        if found:
            hostname = found.group(1)
        return hostname

    @property
    def _base_url(self):
        return "https://www.duckdns.org/update?domains={hostname}" \
               "&token={token}&ip={ip}"


class GenericDnsUpdater(DnsUpdaterPlugin):
    """ Generic DDNS provider plugin - accepts a custom specification for the
    DDNS base url
    """

    auth_type = "P"

    @property
    def _base_url(self):
        return "{url}?hostname={{hostname}}&myip={{ip}}&wildcard=NOCHG" \
               "&mx=NOCHG&backmx=NOCHG".format(url=self._options['url'])



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# noipy.noipy
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

from __future__ import print_function

import argparse
import getpass
import re
import sys

from noipy import utils


from noipy import dnsupdater
from noipy import authinfo
from noipy import __version__


EXECUTION_RESULT_OK = 0
EXECUTION_RESULT_NOK = 1

URL_RE = re.compile(
    r'^https?://'  # http:// or https://
    r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+'
    r'(?:[A-Z]{2,6}\.?|[A-Z0-9-]{2,}\.?)|'  # domain...
    r'localhost|'  # localhost...
    r'\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|'  # ...or ipv4
    r'\[?[A-F0-9]*:[A-F0-9:]+\]?)'  # ...or ipv6
    r'(?::\d+)?'  # optional port
    r'(?:/?|[/?]\S+)$', re.IGNORECASE)


def execute_update(args):
    """Execute the update based on command line args and returns a dictionary
    with 'execution result, ''response code', 'response info' and
    'process friendly message'.
    """

    provider_class = getattr(dnsupdater,
                             dnsupdater.AVAILABLE_PLUGINS.get(args.provider))
    updater_options = {}
    process_message = None
    auth = None

    if args.store:  # --store argument
        if provider_class.auth_type == 'T':
            user_arg = args.usertoken or utils.read_input(
                "Paste your auth token: ")
            auth = authinfo.ApiAuth(usertoken=user_arg)
        else:
            user_arg = args.usertoken or utils.read_input(
                "Type your username: ")
            pass_arg = args.password or getpass.getpass("Type your password: ")
            auth = authinfo.ApiAuth(user_arg, pass_arg)

        authinfo.store(auth, args.provider, args.config)
        exec_result = EXECUTION_RESULT_OK
        if not args.hostname:
            update_ddns = False
            process_message = "Auth info stored."
        else:
            update_ddns = True

    # informations arguments
    elif args.usertoken and args.hostname:
        if provider_class.auth_type == 'T':
            auth = authinfo.ApiAuth(args.usertoken)
        else:
            auth = authinfo.ApiAuth(args.usertoken, args.password)
        update_ddns = True
        exec_result = EXECUTION_RESULT_OK
    elif args.hostname:
        if authinfo.exists(args.provider, args.config):
            auth = authinfo.load(args.provider, args.config)
            update_ddns = True
            exec_result = EXECUTION_RESULT_OK
        else:
            update_ddns = False
            exec_result = EXECUTION_RESULT_NOK
            process_message = "No stored auth information found for " \
                              "provider: '%s'" % args.provider
    else:  # no arguments
        update_ddns = False
        exec_result = EXECUTION_RESULT_NOK
        process_message = "Warning: The hostname to be updated must be " \
                          "provided.\nUsertoken and password can be either " \
                          "provided via command line or stored with --store " \
                          "option.\nExecute noipy --help for more details."

    if update_ddns and args.provider == 'generic':
        if args.url:
            if not URL_RE.match(args.url):
                process_message = "Malformed URL."
                exec_result = EXECUTION_RESULT_NOK
                update_ddns = False
            else:
                updater_options['url'] = args.url
        else:
            process_message = "Must use --url if --provider is 'generic' " \
                              "(default)"
            exec_result = EXECUTION_RESULT_NOK
            update_ddns = False

    response_code = None
    response_text = None
    if update_ddns:
        ip_address = args.ip if args.ip else utils.get_ip()
        if not ip_address:
            process_message = "Unable to get IP address. Check connection."
            exec_result = EXECUTION_RESULT_NOK
        elif ip_address == utils.get_dns_ip(args.hostname):
            process_message = "No update required."
        else:
            updater = provider_class(auth, args.hostname, updater_options)
            print("Updating hostname '%s' with IP address %s "
                  "[provider: '%s']..."
                  % (args.hostname, ip_address, args.provider))
            response_code, response_text = updater.update_dns(ip_address)
            process_message = updater.status_message

    proc_result = {
        'exec_result': exec_result,
        'response_code': response_code,
        'response_text': response_text,
        'process_message': process_message,
    }

    return proc_result


def create_parser():
    parser = argparse.ArgumentParser(
        description="Update DDNS IP address on selected provider.")
    parser.add_argument('-u', '--usertoken', help="provider username or token")
    parser.add_argument('-p', '--password',
                        help="provider password when apply")
    parser.add_argument('-n', '--hostname',
                        help="provider hostname to be updated")
    parser.add_argument('--provider', help="DDNS provider plugin (default: %s)"
                                           % dnsupdater.DEFAULT_PLUGIN,
                        choices=dnsupdater.AVAILABLE_PLUGINS.keys(),
                        default=dnsupdater.DEFAULT_PLUGIN)
    parser.add_argument('--url', help="custom DDNS server address")
    parser.add_argument('--store',
                        help="store DDNS authentication information and "
                             "update the hostname if it is provided",
                        action='store_true')
    parser.add_argument('-c', '--config',
                        help="noipy config directory (default: %s)" %
                             authinfo.DEFAULT_CONFIG_DIR,
                        default=authinfo.DEFAULT_CONFIG_DIR)
    parser.add_argument('ip', metavar='IP_ADDRESS', nargs='?',
                        help="New host IP address. If not provided, current "
                             "external IP address will be used.")

    return parser


def main():
    print("== noipy DDNS updater tool v%s ==" % __version__)
    parser = create_parser()
    args = parser.parse_args()

    result = execute_update(args)
    print(result.get('process_message'))
    if result.get('exec_result') != EXECUTION_RESULT_OK:
        parser.format_usage()

    sys.exit(result.get('exec_result'))


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# noipy.utils
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

import socket

import requests

HTTPBIN_URL = "https://httpbin.org/ip"

try:
    input = raw_input
except NameError:
    pass


def read_input(message):
    return input(message)


def get_ip():
    """Return machine's origin IP address.
    """
    try:
        r = requests.get(HTTPBIN_URL)
        ip, _ = r.json()['origin'].split(',')
        return ip if r.status_code == 200 else None
    except requests.exceptions.ConnectionError:
        return None


def get_dns_ip(dnsname):
    """Return machine's current IP address in DNS.
    """
    try:
        return socket.gethostbyname(dnsname)
    except socket.error:
        return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# setup
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

from setuptools import setup
import sys

from noipy import __version__, __author__, __email__, __license__


install_requires = [
    "requests>=2.0",
]

if sys.version_info[:2] < (2, 7):
    install_requires.append("argparse")


with open("README.rst") as f:
    readme = f.read()
with open("CHANGELOG.rst") as f:
    changelog = f.read()

setup(
    name="noipy",
    version=__version__,
    description="Command line tool for DDNS IP address updating.",
    long_description=readme + "\n\n" + changelog,
    license=__license__,
    author=__author__,
    author_email=__email__,
    url="https://github.com/povieira/noipy",
    packages=["noipy"],
    install_requires=install_requires,
    keywords=["no-ip", "dyndns", "duckdns", "ddns", "api"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: End Users/Desktop",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Internet :: Name Service (DNS)",
    ],
    entry_points={
        "console_scripts": [
            "noipy = noipy.main:main",
        ],
    },
    zip_safe=True,
    test_suite="test.test_noipy"
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# test.test_noipy
# Copyright (c) 2013 Pablo O Vieira (povieira)
# See README.rst and LICENSE for details.

import getpass
import os
import re
import shutil
import unittest

from noipy import authinfo
from noipy import dnsupdater
from noipy import main
from noipy import utils

VALID_IP_REGEX = r"^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25" \
                 r"[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4]" \
                 r"[0-9]|25[0-5])$"


class SanityTest(unittest.TestCase):

    def setUp(self):
        self.parser = main.create_parser()
        self.test_dir = os.path.join(os.path.expanduser("~"), "noipy_test")

    def tearDown(self):
        if os.path.exists(self.test_dir):
            shutil.rmtree(self.test_dir)

    def test_get_ip(self):
        ip = utils.get_ip()

        self.assertTrue(re.match(VALID_IP_REGEX, ip), "get_ip() failed.")

        # monkey patch for testing (forcing ConnectionError)
        utils.HTTPBIN_URL = "http://example.nothing"

        ip = utils.get_ip()
        self.assertTrue(ip is None, "get_ip() should return None. IP=%s" % ip)

    def test_get_dns_ip(self):
        ip = utils.get_dns_ip("localhost")

        self.assertEqual(ip, "127.0.0.1", "get_dns_ip() failed.")

        ip = utils.get_dns_ip("http://example.nothing")
        self.assertTrue(ip is None, "get_dns_ip() should return None. IP=%s"
                        % ip)


class PluginsTest(unittest.TestCase):

    def setUp(self):
        self.parser = main.create_parser()
        self.test_ip = "10.1.2.3"

    def tearDown(self):
        pass

    def test_instance_str(self):
        plugin = dnsupdater.NoipDnsUpdater(auth=None,
                                           hostname="noipy.no-ip.org")
        self.assertEqual(str(plugin), "NoipDnsUpdater(host=noipy.no-ip.org)")

    def test_noip_plugin(self):
        cmd_args = ["-u", "username", "-p", "password",
                    "--provider", "noip",
                    "-n", "noipy.no-ip.org", self.test_ip]

        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Update with 'No-IP' provider failed.")

        # noip.com returns either HTTP 200 or 401,
        # depending on which way the window is blowing
        self.assertTrue(result.get('response_code') in (200, 401),
                        "Invalid response code: %s. Should be 200."
                        % result.get('response_code'))

    def test_dyndns_plugin(self):
        cmd_args = ["-u", "test", "-p", "test",
                    "--provider", "dyn",
                    "-n", "test.dyndns.org", self.test_ip]

        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Update with 'DynDNS' provider failed.")

        self.assertEqual(result.get('response_code'), 200,
                         "Invalid response code: %s. Should be 200."
                         % result.get('response_code'))

    def test_duckdns_plugin(self):
        cmd_args = ["-u", "1234567890ABC",
                    "--provider", "duck",
                    "-n", "noipy.duckdns.org", self.test_ip]

        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Update with 'DuckDNS' provider failed.")

        self.assertEqual(result.get('response_code'), 200,
                         "Invalid response code: %s. Should be 200."
                         % result.get('response_code'))

    def test_generic_plugin(self):
        cmd_args = ["-u", "username", "-p", "password",
                    "--url", "https://dynupdate.no-ip.com/nic/update",
                    "--provider", "generic",
                    "-n", "noipy.no-ip.org", self.test_ip]

        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Update with 'No-IP' using generic provider failed.")

        # noip.com returns either HTTP 200 or 401,
        # depending on which way the window is blowing
        self.assertTrue(result.get('response_code') in (200, 401),
                        "Invalid response code: %s. Should be 200."
                        % result.get('response_code'))

    def test_generic_plugin_malformed_url(self):
        cmd_args = ["-u", "username", "-p", "password",
                    "--url", "abced",
                    "--provider", "generic",
                    "-n", "noipy.no-ip.org", self.test_ip]

        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_NOK,
                         "An error should be flagged when --provider is "
                         "'generic' and URL is malformed.")

        self.assertEqual(result.get('process_message'), "Malformed URL.",
                         "Status message should be an 'Malformed URL.'")

    def test_generic_plugin_without_url(self):
        cmd_args = ["-u", "username", "-p", "password",
                    "--provider", "generic",
                    "-n", "noipy.no-ip.org", self.test_ip]

        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_NOK,
                         "An error should be flagged when --provider is "
                         "'generic' and no URL is provided.")

        self.assertTrue(result.get('process_message')
                        .startswith("Must use --url"),
                        "Status message should start with 'Must use --url'.")


class AuthInfoTest(unittest.TestCase):

    def setUp(self):
        self.parser = main.create_parser()
        self.test_ip = "10.1.2.3"
        self.test_dir = os.path.join(os.path.expanduser("~"), "noipy_test")

    def tearDown(self):
        if os.path.exists(self.test_dir):
            if os.path.isdir(self.test_dir):
                shutil.rmtree(self.test_dir)
            else:
                os.remove(self.test_dir)

    def test_get_instance_password(self):
        auth1 = authinfo.ApiAuth("username", "password")
        auth2 = authinfo.ApiAuth.get_instance(b"dXNlcm5hbWU6cGFzc3dvcmQ=")

        self.assertEqual(auth1, auth2, "ApiAuth.get_instance fail for "
                                       "password.")

    def test_get_token_property_with_password(self):
        auth = authinfo.ApiAuth("username", "password")

        try:
            token = auth.token
            self.fail("A 'NotImplementedError' should be raised. Token=%s" %
                      token)
        except Exception as e:
            if not isinstance(e, NotImplementedError):
                self.fail("A 'NotImplementedError' should be raised. Got: %s"
                          % e)

    def test_get_instance_token(self):
        token = "1234567890ABCDEFG"
        auth1 = authinfo.ApiAuth(usertoken=token)
        auth2 = authinfo.ApiAuth.get_instance(b"MTIzNDU2Nzg5MEFCQ0RFRkc6")

        self.assertEqual(auth1, auth2, "ApiAuth.get_instance fail for token.")
        self.assertEqual(auth1.token, auth2.token, "ApiAuth.token fail.")

    def test_store_from_arguments(self):
        cmd_args = ["--store", "-u", "username", "-p", "password",
                    "--provider", "noip", "-c", self.test_dir]

        # store
        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Error storing auth info")

        self.assertEqual(result.get('process_message'), "Auth info stored.",
                         "Status message should be an 'Auth info stored.'")

        # load
        cmd_args = ["--provider", "noip", "-n", "noipy.no-ip.org",
                    "-c", self.test_dir, self.test_ip]
        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Error loading auth info")

    def test_store_and_perform_update(self):
        cmd_args = ["--store", "-u", "username", "-p", "password",
                    "--provider", "noip", "-n", "noipy.no-ip.org",
                    "-c", self.test_dir, self.test_ip]

        # store
        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Error storing auth info")

        # noip.com returns either HTTP 200 or 401,
        # depending on which way the window is blowing
        self.assertTrue(result.get('response_code') in (200, 401),
                        "Invalid response code: %s. Should be 200."
                        % result.get('response_code'))

    def test_store_from_stdin_input(self):
        cmd_args = ["--store", "--provider", "noip",
                    "-c", self.test_dir]

        # monkey patch for testing
        utils.read_input = lambda _: "username"
        getpass.getpass = lambda _: "password"

        # store
        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Error storing auth info")

        self.assertEqual(result.get('process_message'), "Auth info stored.",
                         "Status message should be an 'Auth info stored.'")

        # load
        cmd_args = ["--provider", "noip", "-n", "noipy.no-ip.org",
                    "-c", self.test_dir, self.test_ip]
        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Error loading auth info")

    def test_store_pass_from_stdin_input(self):
        cmd_args = ["-u", "username", "--store", "--provider", "noip",
                    "-c", self.test_dir]

        # monkey patch for testing
        getpass.getpass = lambda _: "password"

        # store
        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Error storing auth info")

        self.assertEqual(result.get('process_message'), "Auth info stored.",
                         "Status message should be an 'Auth info stored.'")

        # load
        cmd_args = ["--provider", "noip", "-n", "noipy.no-ip.org",
                    "-c", self.test_dir, self.test_ip]
        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Error loading auth info")

    def test_store_token_from_stdin_input(self):
        cmd_args = ["--store", "--provider", "duck",
                    "-c", self.test_dir]

        # monkey patch for testing
        utils.read_input = lambda _: "1234567890ABC"

        # store
        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Error storing auth info")

        self.assertEqual(result.get('process_message'), "Auth info stored.",
                         "Status message should be an 'Auth info stored.'")

        # load
        cmd_args = ["--provider", "duck", "-n", "noipy.duckdns.org",
                    "-c", self.test_dir, self.test_ip]
        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Error loading auth info")

    def test_ioerror(self):
        try:
            auth = authinfo.load(provider="noip",
                                 config_location=self.test_dir)
            self.fail("An 'IOError' should be raised. auth=%s" + auth)
        except Exception as e:
            if not isinstance(e, IOError):
                self.fail("An 'IOError' should be raised. Got: %s" % e)

    def test_update_without_authinfo(self):
        cmd_args = ["--provider", "noip", "-n", "noipy.no-ip.org",
                    "-c", self.test_dir, self.test_ip]

        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_NOK,
                         "Update without auth info failed.")

        self.assertTrue(result.get('process_message').startswith(
            "No stored auth information found for provider:"),
            "Status message should be 'No stored auth information found "
            "for provider: ...'")


class GeneralTest(unittest.TestCase):

    def setUp(self):
        self.parser = main.create_parser()
        self.orig_get_ip = utils.get_ip

    def tearDown(self):
        utils.get_ip = self.orig_get_ip

    def test_cmd_line_no_args(self):
        cmd_args = []

        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_NOK,
                         "Execution without args failed.")

        self.assertTrue(result.get('process_message').startswith(
            "Warning: The hostname to be updated must be provided."),
            "Status message should start with 'Warning: "
            "The hostname to be updated must be provided.'")

    def test_unchanged_ip(self):
        cmd_args = ["-u", "username", "-p", "password",
                    "--url", "https://dynupdate.no-ip.com/nic/update",
                    "--provider", "generic",
                    "-n", "localhost", "127.0.0.1"]

        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_OK,
                         "Update with unchanged IP failed.")

        self.assertEqual(result.get('process_message'), "No update required.",
                         "Status message should be 'No update required'.")

    def test_without_ip(self):
        cmd_args = ["-u", "username", "-p", "password",
                    "--provider", "noip",
                    "-n", "noipy.no-ip.org"]

        # monkey patch for testing
        utils.get_ip = lambda: None

        args = self.parser.parse_args(cmd_args)
        result = main.execute_update(args)

        self.assertEqual(result.get('exec_result'), main.EXECUTION_RESULT_NOK,
                         "Update without IP failed.")

        self.assertTrue(result.get('process_message')
                        .startswith("Unable to get IP address"),
                        "Status message should be 'Unable to get IP address'.")

    def test_not_implemented_plugin(self):
        auth = authinfo.ApiAuth("username", "password")
        hostname = "hostname"
        plugin = dnsupdater.DnsUpdaterPlugin(auth, hostname)
        try:
            plugin.update_dns("10.1.1.1")
            self.fail("Not implemented plugin should fail: "
                      "'NoneType' object has no attribute 'format'")
        except AttributeError as e:
            self.assertEqual(str(e), "'NoneType' object has no attribute "
                                     "'format'",
                             "_get_base_url() should return 'NoneType'")
        except Exception as e:
            self.fail("_get_base_url() should return 'AttributeError'. "
                      "Got %s:%s" % (type(e).__name__, e))

    def test_dns_plugin_status_message(self):
        auth = authinfo.ApiAuth("username", "password")
        hostname = "hostname"
        plugin = dnsupdater.DnsUpdaterPlugin(auth, hostname)

        # badauth code
        plugin.last_ddns_response = "badauth"
        expected_message = "ERROR: Invalid username or password (%s)." \
                           % plugin.last_ddns_response
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'badauth' status code.")

        # good <IP> code
        plugin.last_ddns_response = "good 1.1.1.1"
        expected_message = "SUCCESS: DNS hostname IP (1.1.1.1) successfully " \
                           "updated."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'good <1.1.1.1>' status code.")

        # nochg <IP> code
        plugin.last_ddns_response = "nochg 1.1.1.1"
        expected_message = "SUCCESS: IP address (1.1.1.1) is up to date, " \
                           "nothing was changed. Additional 'nochg' updates " \
                           "may be considered abusive."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'nochg <1.1.1.1>' status code.")

        # !donator code
        plugin.last_ddns_response = "!donator"
        expected_message = "ERROR: Update request include a feature that is " \
                           "not available to informed user."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected '!donator' status code.")

        # notfqdn code
        plugin.last_ddns_response = "notfqdn"
        expected_message = "ERROR: The hostname specified is not a " \
                           "fully-qualified domain name (not in the form " \
                           "hostname.dyndns.org or domain.com)."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'notfqdn' status code.")

        # nohost code
        plugin.last_ddns_response = "nohost"
        expected_message = "ERROR: Hostname specified does not exist in this" \
                           " user account."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'nohost' status code.")

        # numhost code
        plugin.last_ddns_response = "numhost"
        expected_message = "ERROR: Too many hosts (more than 20) specified " \
                           "in an update. Also returned if trying to update " \
                           "a round robin (which is not allowed)."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'numhost' status code.")

        # abuse code
        plugin.last_ddns_response = "abuse"
        expected_message = "ERROR: Username/hostname is blocked due to " \
                           "update abuse."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'abuse' status code.")

        # badagent code
        plugin.last_ddns_response = "badagent"
        expected_message = "ERROR: User agent not sent or HTTP method not " \
                           "permitted."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'badagent' status code.")

        # dnserr code
        plugin.last_ddns_response = "dnserr"
        expected_message = "ERROR: DNS error encountered."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'dnserr' status code.")

        # 911 code
        plugin.last_ddns_response = "911"
        expected_message = "ERROR: Problem on server side. Retry update in a" \
                           " few minutes."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected '911' status code.")

        # OK code
        plugin.last_ddns_response = "OK"
        expected_message = "SUCCESS: DNS hostname successfully updated."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'OK' status code.")

        # KO code
        plugin.last_ddns_response = "KO"
        expected_message = "ERROR: Hostname and/or token incorrect."
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'KO' status code.")

        # Unknown code
        plugin.last_ddns_response = "UNKNOWN_CODE"
        expected_message = "ERROR: Ooops! Something went wrong !!!"
        self.assertEqual(plugin.status_message, expected_message,
                         "Expected 'Ooops' warning message.")


if __name__ == "__main__":
    unittest.main()
